/* This file is part of the KDE project
   Copyright (C) 2002 Lucijan Busch <lucijan@gmx.at>
   Copyright (C) 2003 Joseph Wenninger<jowenn@kde.org>
   Copyright (C) 2004 Jaroslaw Staniek <js@iidea.pl>

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public License
along with this program; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
*/

#ifndef MYSQLCONNECTION_H
#define MYSQLCONNECTION_H

#include <qstringlist.h>

#include <kexidb/connection.h>
#include "mysqlcursor.h"
#include <qdict.h>

namespace KexiDB {

class MySqlConnectionInternal;

/*!
 * Should override kexiDB/kexiDB
 * all other members are done by the
 * base class.
 */
class MySqlConnection : public Connection
{
	Q_OBJECT

	public:
		virtual ~MySqlConnection();

		virtual Cursor* prepareQuery( const QString& statement = QString::null, uint cursor_options = 0 );
		virtual Cursor* prepareQuery( QuerySchema& query, uint cursor_options = 0 );
		virtual QString escapeString( const QString& str) const;
		virtual QCString escapeString( const QCString& str) const;

	protected:

		/*! Used by driver */
		MySqlConnection( Driver *driver, ConnectionData &conn_data );

		virtual bool drv_connect();
		virtual bool drv_disconnect();
		virtual bool drv_getDatabasesList( QStringList &list );
		virtual bool drv_createDatabase( const QString &dbName = QString::null );
		virtual bool drv_useDatabase( const QString &dbName = QString::null );
		virtual bool drv_closeDatabase();
		virtual bool drv_dropDatabase( const QString &dbName = QString::null );
		virtual bool drv_executeSQL( const QString& statement );
		virtual Q_ULLONG drv_lastInsertRowID();

		virtual int serverResult();
		virtual QString serverResultName();
		virtual QString serverErrorMsg();
		virtual void drv_clearServerResult();

//TODO: move this somewhere to low level class (MIGRATION?)
		virtual bool drv_getTablesList( QStringList &list );
//TODO: move this somewhere to low level class (MIGRATION?)
		virtual bool drv_containsTable( const QString &tableName );

		MySqlConnectionInternal* d;

		friend class MySqlDriver;
		friend class MySqlCursor;
};

}

#endif
